@@ -619,10 +619,6 @@ gdk_event_translate (GdkEvent *event,
 	}
 #endif /* G_ENABLE_DEBUG */
 
-      /* bits 13 and 14 in the "state" field are the keyboard group */
-#define KEYBOARD_GROUP_SHIFT 13
-#define KEYBOARD_GROUP_MASK ((1 << 13) | (1 << 14))
-      
       event->key.type = GDK_KEY_PRESS;
       event->key.window = window;
       event->key.time = xevent->xkey.time;
@@ -630,7 +626,11 @@ gdk_event_translate (GdkEvent *event,
       event->key.string = g_strdup (buf);
       event->key.length = charcount;
 
-      event->key.group = (xevent->xkey.state & KEYBOARD_GROUP_MASK) >> KEYBOARD_GROUP_SHIFT;
+      /* bits 13 and 14 in the "state" field are the keyboard group */
+#define KEYBOARD_GROUP_SHIFT 13
+#define KEYBOARD_GROUP_MASK ((1 << 13) | (1 << 14))
+      
+      event->key.group = _gdk_x11_get_group_for_state (xevent->xkey.state);
       
       break;
       
@@ -679,6 +679,8 @@ gdk_event_translate (GdkEvent *event,
       event->key.length = 0;
       event->key.string = NULL;
       
+      event->key.group = (xevent->xkey.state & KEYBOARD_GROUP_MASK) >> KEYBOARD_GROUP_SHIFT;
+
       break;
       
     case ButtonPress:
@@ -716,7 +718,7 @@ gdk_event_translate (GdkEvent *event,
             event->scroll.direction = GDK_SCROLL_RIGHT;
 
 	  event->scroll.window = window;
-	  event->scroll.time = xevent->xbutton.x;
+	  event->scroll.time = xevent->xbutton.time;
 	  event->scroll.x = xevent->xbutton.x + xoffset;
 	  event->scroll.y = xevent->xbutton.y + yoffset;
 	  event->scroll.x_root = (gfloat)xevent->xbutton.x_root;
@@ -1260,9 +1262,8 @@ gdk_event_translate (GdkEvent *event,
         gdk_synthesize_window_state (window,
                                      0,
                                      GDK_WINDOW_STATE_ICONIFIED);
-      
-      if (_gdk_xgrab_window == window_private)
-	_gdk_xgrab_window = NULL;
+
+      _gdk_xgrab_check_unmap (window, xevent->xany.serial);
       
       break;
       
@@ -2047,13 +2048,15 @@ static struct
 } settings_map[] = {
   { "Net/DoubleClickTime", "gtk-double-click-time" },
   { "Net/DndDragThreshold", "gtk-dnd-drag-threshold" },
+  { "Gtk/CanChangeAccels", "gtk-can-change-accels" },
   { "Gtk/ColorPalette", "gtk-color-palette" },
+  { "Gtk/FontName", "gtk-font-name" },
+  { "Gtk/KeyThemeName", "gtk-key-theme-name" },
   { "Gtk/ToolbarStyle", "gtk-toolbar-style" },
   { "Gtk/ToolbarIconSize", "gtk-toolbar-icon-size" },
   { "Net/CursorBlink", "gtk-cursor-blink" },
   { "Net/CursorBlinkTime", "gtk-cursor-blink-time" },
-  { "Net/ThemeName", "gtk-theme-name" },
-  { "Gtk/KeyThemeName", "gtk-key-theme-name" }
+  { "Net/ThemeName", "gtk-theme-name" }
 };
 
 static void
