@@ -212,6 +212,10 @@ void    _gtk_container_clear_resize_widgets   (GtkContainer *container);
 gchar*	_gtk_container_child_composite_name   (GtkContainer *container,
 					      GtkWidget	   *child);
 void   _gtk_container_dequeue_resize_handler (GtkContainer *container);
+GList *_gtk_container_focus_sort             (GtkContainer     *container,
+					      GList            *children,
+					      GtkDirectionType  direction,
+					      GtkWidget        *old_focus);
 
 #ifndef GTK_DISABLE_DEPRECATED
 #define	gtk_container_border_width		gtk_container_set_border_width
