@@ -206,8 +206,11 @@ struct _GtkTextLayoutClass
                                  gint               x,
                                  gint               y);
 
-  GtkFunction pad1;
-  GtkFunction pad2;
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 struct _GtkTextAttrAppearance
