@@ -345,7 +345,7 @@ gtk_tree_path_free (GtkTreePath *path)
  * Return value: A new #GtkTreePath.
  **/
 GtkTreePath *
-gtk_tree_path_copy (GtkTreePath *path)
+gtk_tree_path_copy (const GtkTreePath *path)
 {
   GtkTreePath *retval;
 
@@ -1171,26 +1171,25 @@ gtk_tree_model_foreach_helper (GtkTreeModel            *model,
 			       GtkTreeModelForeachFunc  func,
 			       gpointer                 user_data)
 {
-  gtk_tree_path_append_index (path, 0);
-
   do
     {
       GtkTreeIter child;
 
-      if (gtk_tree_model_iter_children (model, &child, iter))
-	{
-	  if (gtk_tree_model_foreach_helper (model, &child, path, func, user_data))
-	    return TRUE;
-	}
-
       if ((* func) (model, path, iter, user_data))
 	return TRUE;
 
+      if (gtk_tree_model_iter_children (model, &child, iter))
+	{
+	  gtk_tree_path_down (path);
+	  if (gtk_tree_model_foreach_helper (model, &child, path, func, user_data))
+	    return TRUE;
+	  gtk_tree_path_up (path);
+	}
+
       gtk_tree_path_next (path);
     }
   while (gtk_tree_model_iter_next (model, iter));
 
-  gtk_tree_path_up (path);
   return FALSE;
 }
 
@@ -1493,17 +1492,17 @@ disconnect_ref_callbacks (GtkTreeModel *model)
   g_signal_handlers_disconnect_matched (G_OBJECT (model),
                                         G_SIGNAL_MATCH_FUNC,
                                         0, 0, NULL,
-					gtk_tree_row_ref_inserted_callback,
+					(gpointer) gtk_tree_row_ref_inserted_callback,
 					NULL);
   g_signal_handlers_disconnect_matched (G_OBJECT (model),
                                         G_SIGNAL_MATCH_FUNC,
                                         0, 0, NULL,
-					gtk_tree_row_ref_deleted_callback,
+					(gpointer) gtk_tree_row_ref_deleted_callback,
 					NULL);
   g_signal_handlers_disconnect_matched (G_OBJECT (model),
                                         G_SIGNAL_MATCH_FUNC,
                                         0, 0, NULL,
-					gtk_tree_row_ref_reordered_callback,
+					(gpointer) gtk_tree_row_ref_reordered_callback,
 					NULL);
 }
 
